"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Market = exports.MarketBatchAction = exports.getMidmarketPriceNumber = void 0;
const utils_1 = require("@tonic-foundation/utils");
const bn_js_1 = __importDefault(require("bn.js"));
const _1 = require(".");
const batch_1 = require("./batch");
const token_1 = require("./token");
function getMidmarketPriceNumber({ bids, asks, }) {
    if (asks.length && bids.length) {
        const bestAsk = asks[asks.length - 1][0];
        const bestBid = bids[0][0];
        return (bestAsk + bestBid) / 2;
    }
    else if (asks.length) {
        const bestAsk = asks[asks.length - 1][0];
        return bestAsk;
    }
    else if (bids.length) {
        const bestBid = bids[0][0];
        return bestBid;
    }
    return;
}
exports.getMidmarketPriceNumber = getMidmarketPriceNumber;
class MarketBatchAction {
    constructor(market) {
        this.market = market;
        this.batch = new batch_1.BatchActionV1();
    }
    newOrder(params) {
        this.batch.newOrderV1(this.market.id, this.market.toNewOrderParamsV1(params));
        return this;
    }
    cancelOrders(orderIds) {
        this.batch.cancelOrdersV1(this.market.id, orderIds);
        return this;
    }
    cancelAllOrders() {
        this.batch.cancelAllOrdersV1(this.market.id);
        return this;
    }
    prepare() {
        return this.batch.prepare();
    }
}
exports.MarketBatchAction = MarketBatchAction;
class Market {
    constructor(tonic, id, _inner) {
        this.tonic = tonic;
        this.id = id;
        this._inner = _inner;
        this.baseLotSize = _inner.base_token.lot_size;
        this.baseDecimals = _inner.base_token.decimals;
        this.quoteLotSize = _inner.quote_token.lot_size;
        this.quoteDecimals = _inner.quote_token.decimals;
    }
    static load(tonic, id, contractId) {
        return __awaiter(this, void 0, void 0, function* () {
            const view = yield tonic.account.viewFunction(contractId, 'get_market', {
                market_id: id,
            });
            if (!view) {
                throw new Error('market not found');
            }
            return new Market(tonic, id, view);
        });
    }
    /**
     * @ignore
     */
    get inner() {
        return this._inner;
    }
    /**
     * Base taker fee rate in basis points
     */
    get takerFeeBaseRate() {
        return this._inner.taker_fee_base_rate;
    }
    /**
     * Net taker fees accrued in the market (does not include maker/referrer
     * rebates)
     */
    get feesAccrued() {
        return new bn_js_1.default(this._inner.fees_accrued);
    }
    /**
     * Base maker rebate rate in basis points
     */
    get makerRebateBaseRate() {
        return this._inner.maker_rebate_base_rate;
    }
    /**
     * Maximum number of allowed open orders per user account.
     */
    get maxOrdersPerAccount() {
        return this._inner.max_orders_per_account;
    }
    /**
     * Current number of open orders on the market (buys + sells).
     */
    get totalOpenOrders() {
        return this._inner.total_orders;
    }
    get baseTokenId() {
        return (0, token_1.getTokenId)(this._inner.base_token.token_type);
    }
    get quoteTokenId() {
        return (0, token_1.getTokenId)(this._inner.quote_token.token_type);
    }
    get baseTokenType() {
        return this._inner.base_token.token_type;
    }
    get quoteTokenType() {
        return this._inner.quote_token.token_type;
    }
    /**
     * Get minimum price tick as a human readable number.
     */
    get priceTick() {
        return this.priceBnToNumber(new bn_js_1.default(this.quoteLotSize));
    }
    /**
     * Get minimum quantity tick as a human readable number.
     */
    get quantityTick() {
        return this.quantityBnToNumber(new bn_js_1.default(this.baseLotSize));
    }
    /**
     * For internal use
     * @ignore
     */
    toNewOrderParamsV1(params) {
        const order = {
            order_type: params.orderType,
            side: params.side,
            limit_price: params.limitPrice
                ? this.priceNumberToBn(params.limitPrice)
                : undefined,
            quantity: this.quantityNumberToBn(params.quantity),
            max_spend: params.maxSpend
                ? this.priceNumberToBn(params.maxSpend, false, false)
                : undefined,
            client_id: typeof params.clientId === 'undefined' ? null : params.clientId, // potentially 0
        };
        return (0, _1.prepareNewOrderV1)(order);
    }
    placeOrder(params) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.tonic.placeOrder(this.id, this.toNewOrderParamsV1(params));
        });
    }
    /**
     * Same as Tonic.getOrderbook
     */
    getOrderbookRaw(depth) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.tonic.getOrderbook(this.id, depth);
        });
    }
    /**
     * Return the orderbook with prices and quantities converted to human-readable numbers,
     * eg, an order for 0.1 BTC @ 54321.00 USDC will be returned as [54321, 01].
     */
    getOrderbook(depth) {
        return __awaiter(this, void 0, void 0, function* () {
            const orderbook = yield this.tonic.getOrderbook(this.id, depth);
            return this.withNumberValues(orderbook);
        });
    }
    /** @ignore */
    toNumberL2Order(order) {
        return [
            this.priceBnToNumber(order[0]),
            this.quantityBnToNumber(order[1]),
        ];
    }
    /** @ignore */
    withNumberValues(orderbook) {
        // give access to `this` inside the mapped function
        const asks = orderbook.asks.map(this.toNumberL2Order, this);
        const bids = orderbook.bids.map(this.toNumberL2Order, this);
        return { bids, asks };
    }
    createBatchAction() {
        return new MarketBatchAction(this);
    }
    /**
     * Given price as a number, return a decimal-padded BN optionally floored to
     * the nearest quote lot size.
     *
     * If round_trailing_decimals_up===true, the least
     * significant digit + trailing digits are rounded up their ceiling.
     */
    priceNumberToBn(price, floor = true, round_trailing_decimals_up = false) {
        let decimals = this.quoteDecimals;
        let priceBn = (0, utils_1.decimalToBn)(price, decimals);
        if (round_trailing_decimals_up && decimals < 16) {
            // Compare BNs at the highest level of precision available
            // provided by 'number' type (16 digits after decimal point).
            let preciseBn = (0, utils_1.decimalToBn)(price, 16);
            let power = new bn_js_1.default(10).pow(new bn_js_1.default((16 - decimals)));
            if (priceBn.mul(power).lt(preciseBn)) {
                priceBn = priceBn.add(new bn_js_1.default(1));
            }
        }
        if (floor) {
            return (0, utils_1.floorToBn)(priceBn, new bn_js_1.default(this.quoteLotSize));
        }
        return priceBn;
    }
    /**
     * Given price as a number, return an equivalent BN of lots.
     */
    priceNumberToBnLots(price, floor = true) {
        const priceBn = this.priceNumberToBn(price, floor);
        return priceBn.div(new bn_js_1.default(this.quoteLotSize));
    }
    /**
     * Given price as a decimal-padded BN, return a decimal.
     */
    priceBnToNumber(price, precision) {
        return (0, utils_1.bnToApproximateDecimal)(price, this.quoteDecimals, precision);
    }
    /**
     * Given order size as a number, return a decimal-padded BN optionally floored
     * to the nearest base lot size.
     */
    quantityNumberToBn(quantity, floor = true) {
        const qtyBn = (0, utils_1.decimalToBn)(quantity, this.baseDecimals);
        if (floor) {
            return (0, utils_1.floorToBn)(qtyBn, new bn_js_1.default(this.baseLotSize));
        }
        return qtyBn;
    }
    /**
     * Given order size as a number, return an equivalent BN of lots.
     */
    quantityNumberToBnLots(quantity, floor = true) {
        const qtyBn = this.quantityNumberToBn(quantity, floor);
        return qtyBn.div(new bn_js_1.default(this.baseLotSize));
    }
    /**
     * Given quantity as a decimal-padded BN, return a decimal.
     */
    quantityBnToNumber(quantity, precision) {
        return (0, utils_1.bnToApproximateDecimal)(quantity, this.baseDecimals, precision);
    }
}
exports.Market = Market;
