"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMidmarketPrice = exports.groupOrdersByPriceLevel = exports.ZERO = void 0;
const bn_js_1 = __importDefault(require("bn.js"));
exports.ZERO = new bn_js_1.default('0');
function groupOrdersByPriceLevel(orders) {
    return orders.reduce((acc, curr) => {
        if (!acc.length) {
            return [curr];
        }
        const [prev] = acc.slice(-1);
        const [prevPrice, prevSize] = prev;
        const [currPrice, currSize] = curr;
        if (prevPrice.eq(currPrice)) {
            return [
                ...acc.slice(0, -1),
                [prevPrice, prevSize.add(currSize)],
            ];
        }
        return [...acc, curr];
    }, []);
}
exports.groupOrdersByPriceLevel = groupOrdersByPriceLevel;
/**
 * Compute the midmarket price of an orderbook. Price is undefined if only one
 * side has open orders.
 */
function getMidmarketPrice({ asks, bids }) {
    if (asks.length && bids.length) {
        const bestAsk = asks[asks.length - 1][0];
        const bestBid = bids[0][0];
        const midmarketPrice = bestAsk.add(bestBid).divn(2);
        return midmarketPrice;
    }
    else if (asks.length) {
        const bestAsk = asks[asks.length - 1][0];
        return bestAsk;
    }
    else if (bids.length) {
        const bestBid = bids[0][0];
        return bestBid;
    }
    return;
}
exports.getMidmarketPrice = getMidmarketPrice;
